==> 0.2.0.tar.gz: lib/pages/home/slock/mark/AddMark.dart <==
import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/error/BleError.dart';
import 'package:witsystem_core/error/CommandError.dart';
import 'package:witsystem_core/pages/witDialog/DialogCenterInputWidget.dart';
import 'package:witsystem_device_manage/assets/icons/WitsIcon.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_manage/pages/home/slock/widget/MarkWidget.dart';
import 'package:witsystem_device_plugin/command/MarkType.dart';
import 'package:witsystem_device_plugin/device/BleDevice.dart';

///添加指纹
class AddMarkWidget extends StatefulWidget {
  final String deviceName;
  final String deviceId;
  final MarkType markType;

  const AddMarkWidget(
      {Key? key,
      required this.deviceName,
      required this.deviceId,
      required this.markType})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _AddMarkState();
  }
}

class _AddMarkState extends State<AddMarkWidget> with TickerProviderStateMixin {
  AnimationController? _controller;
  Animation<double>? _lineAnimation;
  StreamSubscription? _bleStateListen;
  String? _markName; //指纹名称
  bool _isInput = false; //是否正在录入
  double _totalNumber = 0; //总次数
  double _number = 0; //当前次数
  bool _isUpdateList = false; //上一页是否需要更新列表

  ///点击添加的按键
  _bottom() {
    showDialog(
        context: context,
        barrierDismissible: true,
        builder: (BuildContext context) {
          return DialogCenterInputWidget(
            title: S.current.device_markName,
            contentHint: S.current.device_markNameHint,
            defaultText: _markName,
            outSideDismissible: true,
            onPressed: (bool value, String text) {
              if (value && text.isEmpty) {
                EasyLoading.showToast(S.of(context).device_markNameToast);
                return;
              }
              if (value) {
                _markName = text;
                _addMark();
              }
            },
          );
        });
  }

  ///点击添加指纹
  _addMark() async {
    if (_isInput) {
      return;
    }
    _isInput = true;
    setState(() {});
    EasyLoading.show(status: S.current.device_loading);
    await BleDevice().addMark(widget.deviceId, widget.markType, _markName!,
        commandError: (e) {
      EasyLoading.dismiss();
      _initNumber();
      EasyLoading.showToast(CommandError.error(e["code"]).error,
          duration: const Duration(seconds: 4));
    }, sendSuccess: () {
      EasyLoading.dismiss();
      EasyLoading.showToast(S.of(context).device_inputMarkHint,
          duration: const Duration(seconds: 2));
      _listenBleConnectState();
    }, fail: (e) {

      EasyLoading.dismiss();
      _initNumber();
      EasyLoading.showToast(BleError.error(e["code"]).error,
          duration: const Duration(seconds: 4));
    }, progress: (sum) {
      _totalNumber = double.parse(sum["sum"].toString());
      _number = double.parse(sum["value"].toString());
      if (mounted) {
        setState(() {});
        _controller?.forward();
      }
    }, success: (v) {
      EasyLoading.dismiss();
      EasyLoading.showSuccess(S.current.device_success);
      _markName = "";
      _isUpdateList=true;
      _initNumber();
    });
  }

  ///初始化次数
  _initNumber() {
    _totalNumber = 0;
    _number = 0;
    _isInput = false;
    setState(() {});
  }

  @override
  void initState() {
    super.initState();
    print("获得的类型${widget.markType}");
    _controller = AnimationController(
        duration: const Duration(milliseconds: 120), vsync: this)
      ..addListener(() {
        if (mounted) {
          setState(() {});
        }
      });
    _lineAnimation = Tween(begin: 0.0, end: 1.0).animate(
        CurvedAnimation(curve: const Interval(0.0, 1), parent: _controller!));
    _lineAnimation?.addStatusListener((AnimationStatus status) {
      if (status.name == "completed") {
        _number++; //重置动画需要保持当前的值
        _controller?.reset();
      }
    });
  }

  ///监听和设备的连接状态
  _listenBleConnectState() async {
    //判断已经监听无须重复监听，防止重复监听
    if (_bleStateListen != null) {
      return;
    }
    Stream _stream = await BleDevice().bleDeviceConnectState();
    _bleStateListen = _stream.listen((value) {
      _cancelListenBleConnectState();
      _initNumber();
    });
  }

  ///取消监听状态
  _cancelListenBleConnectState() {
    if (_bleStateListen != null) _bleStateListen?.cancel();
    _bleStateListen = null;
  }

  @override
  void dispose() {
    super.dispose();
    _controller?.dispose();
    EasyLoading.dismiss();
    BleDevice().cancelAddMark(widget.deviceId);
    _cancelListenBleConnectState();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        onWillPop: (){
          Navigator.pop(context,_isUpdateList);
          return Future.value(false);
        },
        child: Scaffold(
          backgroundColor: Theme.of(context).primaryColor,
          appBar: AppBar(
            title:
                Text(S.of(context).device_addMark + "(${widget.deviceName})"),
          ),
          body: Flex(
            mainAxisSize: MainAxisSize.max,
            direction: Axis.vertical,
            children: [
              Expanded(
                  flex: 2,
                  child: Container(
                      alignment: Alignment.center,
                      child: Stack(
                        alignment: Alignment.center,
                        children: [
                          MarkWidget(
                            width: 200,
                            height: 200,
                            progressBackground: Colors.black12,
                            progressColor: Colors.white,
                            totalProgress: _totalNumber,
                            progress: (_number - 1 + _lineAnimation!.value),
                          ),
                          const Icon(WitsIcon.zhiwen,
                              size: 50, color: Colors.white),
                          Container(
                            width: 50,
                            height: 50,
                            alignment: Alignment.topCenter,
                            child: Container(
                                width: 50,
                                height: _totalNumber == 0
                                    ? 50
                                    : 50 *
                                        (1 -
                                            (_number -
                                                    1 +
                                                    _lineAnimation!.value) /
                                                _totalNumber),
                                color: const Color(0xc02D306C)),
                          ),
                        ],
                      ))),
              Expanded(
                  flex: 1,
                  child: Container(
                    alignment: Alignment.center,
                    child: MaterialButton(
                      enableFeedback: false,
                      minWidth: 160,
                      height: 50,
                      color: !_isInput
                          ? Theme.of(context).primaryColorLight
                          : Colors.black12,
                      onPressed: _bottom,
                      child: Text(
                        S.of(context).device_addMark,
                        style: TextStyle(
                            color: !_isInput ? Colors.white : Colors.white38),
                      ),
                    ),
                  ))
            ],
          ),
        ));
  }
}

==> 0.2.8.tar.gz: lib/pages/home/slock/mark/AddMark.dart <==
import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:witsystem_core/error/BleError.dart';
import 'package:witsystem_core/error/CommandError.dart';
import 'package:witsystem_core/error/handle/AutoErrorHandle.dart';
import 'package:witsystem_core/pages/witDialog/DialogCenterInputWidget.dart';
import 'package:witsystem_device_manage/assets/icons/WitsIcon.dart';
import 'package:witsystem_device_manage/generated/l10n.dart';
import 'package:witsystem_device_manage/pages/home/slock/widget/MarkWidget.dart';
import 'package:witsystem_device_plugin/command/MarkType.dart';
import 'package:witsystem_device_plugin/device/BleDevice.dart';

///添加指纹
class AddMarkWidget extends StatefulWidget {
  final String deviceName;
  final String deviceId;
  final MarkType markType;

  const AddMarkWidget(
      {Key? key,
      required this.deviceName,
      required this.deviceId,
      required this.markType})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _AddMarkState();
  }
}

class _AddMarkState extends State<AddMarkWidget> with TickerProviderStateMixin {
  AnimationController? _controller;
  Animation<double>? _lineAnimation;
  StreamSubscription? _bleStateListen; //蓝牙连接状态监听对象
  String? _markName; //指纹名称
  bool _isInput = false; //是否正在录入
  double _totalNumber = 0; //总次数
  double _number = 0; //当前次数
  bool _isUpdateList = false; //上一页是否需要更新列表

  ///点击添加的按键
  _bottom() {
    showDialog(
        context: context,
        barrierDismissible: true,
        builder: (BuildContext context) {
          return DialogCenterInputWidget(
            title: S.current.device_markName,
            contentHint: S.current.device_markNameHint,
            defaultText: _markName,
            outSideDismissible: true,
            onPressed: (bool value, String text) {
              if (value && text.isEmpty) {
                EasyLoading.showToast(S.of(context).device_markNameToast);
                return;
              }
              if (value) {
                _markName = text;
                _addMark();
              }
            },
          );
        });
  }

  ///点击添加指纹
  _addMark() async {
    if (_isInput) {
      return;
    }
    _isInput = true;
    setState(() {});
    EasyLoading.show(status: S.current.device_loading);
    await BleDevice().addMark(widget.deviceId, widget.markType, _markName!,
        sendSuccess: () {
      EasyLoading.dismiss();
      EasyLoading.showToast(S.of(context).device_inputMarkHint,
          duration: const Duration(seconds: 2));
      _listenBleConnectState();
    }, progress: (sum) {
      print("进度${(sum)}");
      _totalNumber = double.parse(sum["sum"].toString());
      _number = double.parse(sum["value"].toString());
      if (mounted) {
        setState(() {});
        _controller?.forward();
      }
    }).then((value) {
      EasyLoading.dismiss();
      EasyLoading.showSuccess(S.current.device_success);
      _markName = "";
      _isUpdateList = true;
      _initNumber();
    }).onError((error, stackTrace) {
      final e = error as PlatformException;
      EasyLoading.dismiss();
      _initNumber();
      EasyLoading.showToast(AutoErrorHandle.auto(e).error,
          duration: const Duration(seconds: 4));
    });
  }

  ///初始化次数
  _initNumber() {
    _totalNumber = 0;
    _number = 0;
    _isInput = false;
    setState(() {});
  }

  @override
  void initState() {
    super.initState();
    print("获得的类型${widget.markType}");
    _controller = AnimationController(
        duration: const Duration(milliseconds: 120), vsync: this)
      ..addListener(() {
        if (mounted) {
          setState(() {});
        }
      });
    _lineAnimation = Tween(begin: 0.0, end: 1.0).animate(
        CurvedAnimation(curve: const Interval(0.0, 1), parent: _controller!));
    _lineAnimation?.addStatusListener((AnimationStatus status) {
      if (status.name == "completed") {
        _number++; //重置动画需要保持当前的值
        _controller?.reset();
      }
    });
  }

  ///监听和设备的连接状态
  _listenBleConnectState() async {
    //判断已经监听无须重复监听，防止重复监听
    if (_bleStateListen != null) {
      return;
    }
    _bleStateListen = BleDevice().bleDeviceConnectState((value) {
      _cancelListenBleConnectState();
      _initNumber();
    });
  }

  ///取消监听状态
  _cancelListenBleConnectState() {
    if (_bleStateListen != null)
      BleDevice().cancelBleDeviceConnectState(subscription: _bleStateListen);
    _bleStateListen = null;
  }

  @override
  void dispose() {
    super.dispose();
    _controller?.dispose();
    EasyLoading.dismiss();
    BleDevice().cancelAddMark(widget.deviceId);
    _cancelListenBleConnectState();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        onWillPop: () {
          Navigator.pop(context, _isUpdateList);
          return Future.value(false);
        },
        child: Scaffold(
          backgroundColor: Theme.of(context).primaryColor,
          appBar: AppBar(
            title:
                Text(S.of(context).device_addMark + "(${widget.deviceName})"),
          ),
          body: Flex(
            mainAxisSize: MainAxisSize.max,
            direction: Axis.vertical,
            children: [
              Expanded(
                  flex: 2,
                  child: Container(
                      alignment: Alignment.center,
                      child: Stack(
                        alignment: Alignment.center,
                        children: [
                          MarkWidget(
                            width: 200,
                            height: 200,
                            progressBackground: Colors.black12,
                            progressColor: Colors.white,
                            totalProgress: _totalNumber,
                            progress: (_number - 1 + _lineAnimation!.value),
                          ),
                          const Icon(WitsIcon.zhiwen,
                              size: 50, color: Colors.white),
                          Container(
                            width: 50,
                            height: 50,
                            alignment: Alignment.topCenter,
                            child: Container(
                                width: 50,
                                height: _totalNumber == 0
                                    ? 50
                                    : 50 *
                                        (1 -
                                            (_number -
                                                    1 +
                                                    _lineAnimation!.value) /
                                                _totalNumber),
                                color: const Color(0xc02D306C)),
                          ),
                        ],
                      ))),
              Expanded(
                  flex: 1,
                  child: Container(
                    alignment: Alignment.center,
                    child: MaterialButton(
                      enableFeedback: false,
                      minWidth: 160,
                      height: 50,
                      color: !_isInput
                          ? Theme.of(context).primaryColorLight
                          : Colors.black12,
                      onPressed: _bottom,
                      child: Text(
                        S.of(context).device_addMark,
                        style: TextStyle(
                            color: !_isInput ? Colors.white : Colors.white38),
                      ),
                    ),
                  ))
            ],
          ),
        ));
  }
}
